<?php

namespace App\Http\Controllers;

use App\Interfaces\IMenu;
use App\Interfaces\IPrivilage;
use App\Interfaces\IRole;
use App\Interfaces\IUser;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class PrivilegeController extends Controller
{

    private $privileges, $roles, $menu, $user;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(IPrivilage $privileges, IRole $role, IMenu $menu, IUser $user)
    {
        $this->privileges = $privileges;
        $this->roles = $role;
        $this->menu = $menu;
        $this->user = $user;
    }


    public function index()
    {
        $privileges = $this->privileges->selectItems(['active_flag'=> 1], ['id','name', 'create', 'read', 'update', 'delete', 'active_flag']);

        return $this->sendResponse($privileges, 'success');
    }

    public function allPrivileges()
    {
        $privileges = $this->privileges->selectAll(['name', 'create', 'id', 'read', 'update', 'delete', 'active_flag']);

        return $this->sendResponse($privileges, 'success');
    }

    public function createPrivilege(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "name" => "required|unique:privileges",
            "create" => "required|boolean",
            "read" => "required|boolean",
            "update" => "required|boolean",
            "delete" => "required|boolean",
        ]);

        if ($validator->fails()) {
            return $this->sendError('Validation Error- ' . $validator->errors()->first(), $validator->errors());
        }

        $this->privileges->create([
            'name'=> $request->get('name'),
            'create'=> $request->get('create'),
            'read'=> $request->get('read'),
            'update'=> $request->get('update'),
            'delete'=> $request->get('delete')
        ]);

        return $this->sendResponse($this->privileges->new_insert(['id', 'name', 'create', 'read', 'update', 'delete']), "Privilege created successfully");
    }

    public function updatePrivilege(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'name'=> 'required'
        ]);

        if ($validator->fails()) {
            return $this->sendError('Validation Error- ' . $validator->errors()->first(), $validator->errors());
        }

        $privilege = $this->privileges->findItem(['id'=> $request->segment(3)]);

        if(is_null($privilege)){
            return $this->sendError('Invalid privilege id');
        }

        if($privilege->active_flag == 0){
            return $this->sendError('Privilege is currently disable');
        }

        $this->privileges->updateItem(['id'=> $privilege->id], [
            'name'=> $request->get('name')
        ]);

        $privilege->name = $request->get('name');

        return $this->sendResponse($privilege, 'Privilege updated successfully');


    }


    public function enablePrivilege(Request $request)
    {
        $privilege = $this->privileges->findItem((['id'=> $request->segment(3)]));

        if(is_null($privilege)){
            return $this->sendError("Invalid Privilege Id");
        }

        if($privilege->active_flag == 0){
            $this->privileges->updateItem(['id'=> $privilege->id], ['active_flag'=> 1]);
        }

        $privilege->active_flag = 1;

        return $this->sendResponse("Privilege enabled successfully", $privilege);
    }

    public function disablePrivilege(Request $request)
    {
        $privilege = $this->privileges->findItem((['id' => $request->segment(3)]));

        if (is_null($privilege)) {
            return $this->sendError("Invalid Privilege Id");
        }

        if ($privilege->active_flag == 1) {
            $this->privileges->updateItem(['id' => $privilege->id], ['active_flag' => 1]);
        }

        $privilege->active_flag = 1;

        return $this->sendResponse("Privilege disabled successfully", $privilege);
    }

    public function assignPrivileges(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'role'=> 'required|string',
            "permissions"=> 'required|array',
            'permission.*'=> 'required|array',
        ]);

        if($validator->fails()){
            return $this->sendError('Validation Error- ' . $validator->errors()->first(), $validator->errors());
        }

        $role = $this->roles->findItem(['role_slug'=> $request->get('role')]);

        if(is_null($role)){
            return $this->sendError('Invalid role slug');
        }

        if($role->active_flag == 0){
            return $this->sendError('Role is not active');
        }

        $role_permissions = [];

        $error_message = "The following actions not found or active ";
        $error_count = 0;
        //dd($request->get('permissions'));

        foreach($request->get('permissions') as $action => $permissions){
            //dd($permissions);
            foreach($permissions as  $key => $value){
                $db_action =  $this->privileges->findItem(["name" => $key]);
                // $db_action =  $this->privileges->findItem(["name" => $action]);
                if ($db_action == null) {
                    $error_message .= $action . ", ";
                    $error_count++;
                } else if ($db_action->active_flag == 0) {
                    $error_message .= $action . ", ";
                    $error_count++;
                } else {
                    $role_permissions[] = (object) [
                        "id" => $db_action->id,
                        "create" => $db_action->create == 1 ?
                            (isset($value["create"]) ?
                                $value["create"] : 0) : 0,
                        "read" => $db_action->read == 1 ?
                            (isset($value["read"]) ?
                                $value["read"] : 0) : 0,
                        "update" => $db_action->update == 1 ?
                            (isset($value["update"]) ?
                                $value["update"] : 0) : 0,
                        "delete" => $db_action->delete == 1 ?
                            (isset($value["delete"]) ?
                                $value["delete"] : 0) : 0
                    ];
                }
               
            }
        }

        if($error_count > 0){
            return $this->sendError('Permission error', $error_message);
        }

        foreach ($role_permissions as $role_permission) {
            $this->privileges->assignPrivilegesToRoles($role_permission, $role);
        }

        $role_name = $role->name;

        return $this->sendResponse('', "Privileges for {$role_name} has been assigned successfully");
    }

    public function getUserMenu(Request $request)
    {
        $user = $request->user();
        // $user = $id;

        //return $this->sendResponse($user, 'success');
        // $user = $this->user->findItem(['id'=>$id]);
        // return $user;
        // if ($user->super_user == 1) {
        if ($user->super_user == 1) {
            
            $menus = $this->menu->getAllActiveMenus();
        } else {
           
            $menus = $this->menu->getUserMenus($user);
        }

       

        return $this->sendResponse($menus, 'success');
    }
}
